/**
 * vim: set ts=4 :
 * =============================================================================
 * SourceMod Sample Extension
 * Copyright (C) 2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "extension.h"
#include <ihltv.h>
#include <ihltvdirector.h>
#include <iserver.h>


SH_DECL_HOOK0(IServer, GetMapName, const, false, const char *);

QueryFix g_QueryFix;
SMEXT_LINK(&g_QueryFix);

IHLTVDirector *hltvdirector = NULL;
int g_HookId = 0;

bool QueryFix::SDK_OnLoad(char *error, size_t maxlength, bool late)
{
	playerhelpers->AddClientListener(this);

	if (late)
	{
		for (int i = 1; i <= playerhelpers->GetMaxClients(); i++)
		{
			IGamePlayer *pPlayer = playerhelpers->GetGamePlayer(i);

			if (pPlayer && pPlayer->IsConnected())
				OnClientConnected(i);
		}
	}

	return true;
}

bool QueryFix::SDK_OnMetamodLoad(ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
	GET_V_IFACE_CURRENT(GetServerFactory, hltvdirector, IHLTVDirector, INTERFACEVERSION_HLTVDIRECTOR);

	return true;
}

void QueryFix::SDK_OnUnload()
{
	playerhelpers->RemoveClientListener(this);

	if (g_HookId != 0)
	{
		SH_REMOVE_HOOK_ID(g_HookId);
		g_HookId = 0;
	}
}

void QueryFix::OnClientConnected(int client)
{
	if (g_HookId != 0)
		return;

	IGamePlayer *pPlayer = playerhelpers->GetGamePlayer(client);

	if (!pPlayer || !pPlayer->IsSourceTV())
		return;

	for (int i = 0; i < hltvdirector->GetHLTVServerCount(); i++)
	{
		IHLTVServer *hltvserver = hltvdirector->GetHLTVServer(i);

		if (hltvserver && hltvserver->GetBaseServer())
		{
			g_HookId = SH_ADD_VPHOOK(IServer, GetMapName, hltvserver->GetBaseServer(), SH_MEMBER(this, &QueryFix::Hook_GetMapName), false);
			break;
		}
	}
}

const char *QueryFix::Hook_GetMapName() const
{
	const char *map = gamehelpers->GetCurrentMap();

	if (map)
		RETURN_META_VALUE(MRES_SUPERCEDE, map);

	RETURN_META_VALUE(MRES_IGNORED, NULL);
}

